﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TicTacToe.Model.Model;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Model.UnitTest.ModelTests
{
    [TestClass]
    public class OuterFieldTest
    {
        #region HasCompletedLineTests       

        [TestMethod]
        public void HasCompletedLineCenterColWinLastRowSetXReturnTrue()
        {
            var makeStepIn = 7;
            var outerField = new OuterField().Fill(
                            @"-x-
                              -x-
                              ---");

            outerField[makeStepIn].MakeX();

            var actual = outerField.HasCompletedLine(makeStepIn, true);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineLeftColWinLastRowSetXReturnTrue()
        {
            var makeStepIn = 6;
            var outerField = new OuterField().Fill(
                            @"xx-
                              xo-
                              --o");

            outerField[makeStepIn].MakeX();

            var actual = outerField.HasCompletedLine(makeStepIn, true);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineRightColWinMidRowSetXReturnTrue()
        {
            var makeStepIn = 5;
            var outerField = new OuterField().Fill(
                            @"xox
                              xo-
                              --x");

            outerField[makeStepIn].MakeX();

            var actual = outerField.HasCompletedLine(makeStepIn, true);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineRightColMidRowSetXReturnFalse()
        {
            var makeStepIn = 5;
            var outerField = new OuterField().Fill(
                            @"xox
                              xo-
                              --o");

            outerField[makeStepIn].MakeX();

            var actual = outerField.HasCompletedLine(makeStepIn, true);
            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void HasCompletedLineBotRowWinLeftColSetOReturnTrue()
        {
            var makeStepIn = 6;
            var outerField = new OuterField().Fill(
                            @"xox
                              xo-
                              -oo");

            outerField[makeStepIn].MakeO();

            var actual = outerField.HasCompletedLine(makeStepIn, false);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineMidRowWinCenterColSetOReturnTrue()
        {
            var makeStepIn = 4;
            var outerField = new OuterField().Fill(
                            @"xox
                              o-o
                              -хo");

            outerField[makeStepIn].MakeO();

            var actual = outerField.HasCompletedLine(makeStepIn, false);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineDiagLeftWinRightColSetOReturnTrue()
        {
            var makeStepIn = 0;
            var outerField = new OuterField().Fill(
                            @"-ox
                              xoo
                              xхo");

            outerField[makeStepIn].MakeO();

            var actual = outerField.HasCompletedLine(makeStepIn, false);
            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void HasCompletedLineDiagRightWinRightColSetOReturnTrue()
        {
            var makeStepIn = 6;
            var outerField = new OuterField().Fill(
                            @"-oo
                              xoo
                              -хo");

            outerField[makeStepIn].MakeO();

            var actual = outerField.HasCompletedLine(makeStepIn, false);
            Assert.IsTrue(actual);
        }

        #endregion

        #region CanSomeoneWin Tests

        [TestMethod]
        public void CanSomeoneWinRowCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"xo-
                                                 xx-
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinEmptyRowCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"xo-
                                                 xxo
                                                 ---");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinRowCanNotBeWonReturnFalse()
        {
            var target = new OuterField().Fill(@"xox
                                                 xoo
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void CanSomeoneWinColCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"xo-
                                                 xox
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinEmptyColCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"xo-
                                                 xo-
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinColCanNotBeWonReturnFalse()
        {
            var target = new OuterField().Fill(@"xo-
                                                 oxx
                                                 oxo");
            var actual = target.CanSomeoneWin();

            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void CanSomeoneWinLeftDiagCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"oox
                                                 xoo
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinEmptyLeftDiagCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"-ox
                                                 x-o
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinLeftDiagCanNotBeWonReturnFalse()
        {
            var target = new OuterField().Fill(@"oox
                                                 xxo
                                                 ox-");
            var actual = target.CanSomeoneWin();

            Assert.IsFalse(actual);
        }

        [TestMethod]
        public void CanSomeoneWinRightDiagCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"ox-
                                                 x-o
                                                 oox");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinEmptyRightDiagCanBeWonReturnTrue()
        {
            var target = new OuterField().Fill(@"xo-
                                                 o-x
                                                 -xo");
            var actual = target.CanSomeoneWin();

            Assert.IsTrue(actual);
        }

        [TestMethod]
        public void CanSomeoneWinRightDiagCanNotBeWonReturnFalse()
        {
            var target = new OuterField().Fill(@"ox-
                                                 xxo
                                                 oox");
            var actual = target.CanSomeoneWin();

            Assert.IsFalse(actual);
        }

        #endregion
    }
}
