﻿using TicTacToe.Model.Interfaces;

namespace TicTacToe.Model.Model
{
    public abstract class FieldBase : IField
    {
        public IFieldItem[] Fields { get; set; }

        public bool HasCompletedLine(int lastIndex, bool isLastStepX)
        {
            const int n = 3;
            int lastIndexX = 0;
            int lastIndexY = 0;

            var board = new IFieldItem[n, n];
            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                {
                    var innerFieldIndex = i * n + j;
                    if (innerFieldIndex == lastIndex)
                    {
                        lastIndexX = i;
                        lastIndexY = j;
                    }
                    board[i, j] = Fields[innerFieldIndex];
                }
            }

            //check col
            for (int i = 0; i < n; i++)
            {
                if (isLastStepX ? !board[lastIndexX, i].IsX : !board[lastIndexX, i].IsO)
                    break;
                if (i == n - 1)
                {
                    //report win for lastIndexState
                    return true;
                }
            }

            //check row
            for (int i = 0; i < n; i++)
            {
                if (isLastStepX ? !board[i, lastIndexY].IsX : !board[i, lastIndexY].IsO)
                    break;
                if (i == n - 1)
                {
                    //report win for lastIndexState
                    return true;
                }
            }

            //check diag
            if (lastIndexX == lastIndexY)
            {
                //we're on a diagonal
                for (int i = 0; i < n; i++)
                {
                    if (isLastStepX ? !board[i, i].IsX : !board[i, i].IsO)
                        break;
                    if (i == n - 1)
                    {
                        //report win for lastIndexState
                        return true;
                    }
                }
            }

            //check anti diag (thanks rampion)
            for (int i = 0; i < n; i++)
            {
                if (isLastStepX ? !board[i, (n - 1) - i].IsX : !board[i, (n - 1) - i].IsO)
                    break;
                if (i == n - 1)
                {
                    //report win for lastIndexState
                    return true;
                }
            }

            return false;
        }

        public bool CanSomeoneWin()
        {
            bool canWinRow = false;
            bool canWinCol = false;
            bool leftDiagHasX = false;
            bool leftDiagHasO = false;

            bool rightDiagHasX = false;
            bool rightDiagHasO = false;
            
            for (int i = 0; i < 3; i++)
            {
                bool rowHasX = false;
                bool rowHasO = false;

                bool colHasX = false;
                bool colHasO = false;

                for (int j = 0; j < 3; j++)
                {
                    rowHasO |= Fields[i * 3 + j].IsO;
                    rowHasX |= Fields[i * 3 + j].IsX;

                    colHasO |= Fields[j * 3 + i].IsO;
                    colHasX |= Fields[j * 3 + i].IsX;
                }

                leftDiagHasO |= Fields[i * 4].IsO;
                leftDiagHasX |= Fields[i * 4].IsX;

                rightDiagHasO |= Fields[(i + 1) * 2].IsO;
                rightDiagHasX |= Fields[(i + 1) * 2].IsX;

                canWinRow |= !(rowHasO && rowHasX);
                canWinCol |= !(colHasO && colHasX);
            }

            bool canWinLeftDiagonal = !(leftDiagHasO && leftDiagHasX);
            bool canWinRightDiagonal = !(rightDiagHasO && rightDiagHasX);

            return canWinRow || canWinCol || canWinLeftDiagonal || canWinRightDiagonal;
        }
    }
}


