﻿using System;
using System.Data;
using System.Linq;
using TicTacToe.Common.Log;
using TicTacToe.Model.Interfaces;
using TicTacToe.Model.Model;

namespace TicTacToe.Model.Repositories
{
    public class StatisticsRepository : IStatisitcsRepository
    {
        private readonly ILoggerFacade logger;

        public StatisticsRepository()
        {
            logger = new LoggerFacade(GetType());
        }

        public Statistics GetUserStatistics(Guid userId)
        {
            using (var dbContext = new TicTacToeContext())
            {
                return dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);
            }
        }

        public void AddWonGames(Guid userId, int amount)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);

                if (statistics == null)
                {
                    statistics = new Statistics { UserId = userId, Id = Guid.NewGuid() };
                    dbContext.Statistics.Add(statistics);
                }

                statistics.WonGames += amount;
                dbContext.SaveChanges();
            }
        }

        public void AddLostGames(Guid userId, int amount)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);

                if (statistics == null)
                {
                    statistics = new Statistics { UserId = userId, Id = Guid.NewGuid() };
                    dbContext.Statistics.Add(statistics);
                }

                statistics.LostGames += amount;
                dbContext.SaveChanges();
            }
        }

        public void AddGamesPlayed(Guid userId, int amount)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);

                if (statistics == null)
                {
                    statistics = new Statistics { UserId = userId, Id = Guid.NewGuid() };
                    dbContext.Statistics.Add(statistics);
                }

                statistics.GamesPlayed += amount;
                
                dbContext.SaveChanges();
            }
        }

        public void AddDrawGames(Guid userId, int amount)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);

                if (statistics == null)
                {
                    statistics = new Statistics { UserId = userId, Id = Guid.NewGuid() };
                    dbContext.Statistics.Add(statistics);
                }

                statistics.DrawGames += amount;
                dbContext.SaveChanges();
            }
        }

        public void AddLeaves(Guid userId, int amount)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.UserId == userId);

                if (statistics == null)
                {
                    statistics = new Statistics { UserId = userId, Id = Guid.NewGuid() };
                    dbContext.Statistics.Add(statistics);
                }

                statistics.Leaves += amount;
                dbContext.SaveChanges();
            }
        }

        public void MoveStatisticToNewUser(Guid statisticsId, Guid toUserId)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var statistics = dbContext.Statistics.FirstOrDefault(s => s.Id == statisticsId);

                if (statistics != null)
                {
                    statistics.UserId = toUserId;
                    dbContext.SaveChanges();
                }
            }
        }
    }
}
