﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using TicTacToe.Common.Log;
using TicTacToe.Model.Interfaces;
using TicTacToe.Model.Model;

namespace TicTacToe.Model.Repositories
{
    public class UserRepository : IUserRepository
    {
        private readonly ILoggerFacade logger;

        public UserRepository()
        {
            logger = new LoggerFacade(this.GetType());
        }
        
        public IEnumerable<User> GetAllUsers()
        {
            using (var dbContext = new TicTacToeContext())
            {
                return dbContext.Users.Select(x => x);
            }
        }

        public User GetUserById(Guid id)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var user = dbContext.Users.FirstOrDefault(u => u.Id == id);
                if (user != null)
                {
                    return user;
                }

                logger.Error("User with id => " + id + " not found.");
                throw new ObjectNotFoundException("User with id => " + id + " not found.");
            }
        }

        public User GetUserByEmail(string email)
        {
            using (var dbContext = new TicTacToeContext())
            {
                var user = dbContext.Users.FirstOrDefault(u => u.Email == email);
                if (user != null)
                {
                    return user;
                }

                logger.Error("User with email => " + email + " not found.");
                throw new ObjectNotFoundException("User with email => " + email + " not found.");
            }
        }

        public void CreateUser(User user)
        {
            using (var dbContext = new TicTacToeContext())
            {
                try
                {
                    dbContext.Users.Add(user);
                    dbContext.SaveChanges();
                }
                catch (Exception ex)
                {
                    logger.Error(ex.Message);
                    throw;
                }
            }
        }

        public bool IsUserExists(string email)
        {
            using (var dbContext = new TicTacToeContext())
            {
                return dbContext.Users.Any(x => x.Email.Equals(email, StringComparison.OrdinalIgnoreCase));
            }
        }

        public bool IsUserExists(Guid id)
        {
            using (var dbContext = new TicTacToeContext())
            {
                return dbContext.Users.Any(x => x.Id == id);
            }
        }
    }
}
