﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TicTacToe.Service.Communication;
using System.Threading;

namespace TicTacToe.Service.UnitTest.CommunicationTests
{
    [TestClass]
    public class PlayerConnectionTest
    {
        [TestMethod]
        public void CtorAllSpecifiedObjectCreated()
        {
            var target = new PlayerConnection(Guid.NewGuid());

            Assert.IsNotNull(target);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void CtorPlayerIdIsDefaultThrowException()
        {
            var target = new PlayerConnection(default(Guid));            
        }

        [TestMethod]
        public void LastMessageArrivedObjectCreatedIsNotDefault()
        {
            var target = new PlayerConnection(Guid.NewGuid());

            var actual = target.LastMessageArrived;

            Assert.AreNotEqual(default(DateTime), actual);
        }

        [TestMethod]
        public void MessageArrivedMethodCalledLastMessageArrivedUpdated()
        {
            var target = new PlayerConnection(Guid.NewGuid());
            var oldValue = target.LastMessageArrived;
            
            Thread.Sleep(100);
            target.MessageArrived();
           
            var actual = target.LastMessageArrived;

            Assert.AreNotEqual(oldValue.ToFileTimeUtc(), actual.ToFileTimeUtc());
        }

        [TestMethod]
        public void IdObjectCreatedSetToConstructoParamValue()
        {
            var expected = Guid.NewGuid();
            var target = new PlayerConnection(expected);

            var actual = target.Id;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void DisconnectedCallMethodRaisePlayerDisconnectedEvent()
        {
            var target = new PlayerConnection(Guid.NewGuid());
            var isEventRasied = false;
            target.PlayerDisconnected += (sender, args) =>
            {
                isEventRasied = true;
            };

            target.Disconnect();

            Assert.IsTrue(isEventRasied);
        }
    }
}
