﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

using TicTacToe.Model.Model;
using TicTacToe.Service.DataContracts;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Service.UnitTest.TicTacToeServiceTests
{
    [TestClass]
    public class CheckInitGameStatus
    {
        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusOnePlayerInitiatedReturnWaitForOpponent()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitGame();

            var actual = target.CheckInitGameStatus(gameInitInfo);

            Assert.AreEqual(CheckGameInitStatus.WaitForOpponent, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusWrongGameIdReturnWrongGameId()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            target.InitGame();

            var actual = target.CheckInitGameStatus(new GameAndPlayerInfo { GameId = Guid.NewGuid() });

            Assert.AreEqual(CheckGameInitStatus.WrongGameId, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusWrongPlayerIdReturnWrongPlayerId()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitGame();

            var actual = target.CheckInitGameStatus(new GameAndPlayerInfo
                                                        {
                                                            GameId = gameInitInfo.GameId,
                                                            PlayerId = Guid.NewGuid()
                                                        });

            Assert.AreEqual(CheckGameInitStatus.WrongPlayerId, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusTwoPlayerInitiatedReturnGameInitiated()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitGame();
            target.InitGame();

            var actual = target.CheckInitGameStatus(gameInitInfo);

            Assert.AreEqual(CheckGameInitStatus.GameInitiated, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusOnePlayerInitiatedFriendGameReturnWaitForOpponent()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitFriendGame();

            var actual = target.CheckInitGameStatus(gameInitInfo);

            Assert.AreEqual(CheckGameInitStatus.WaitForOpponent, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusOnePlayerInitiatedAndOneConnectedFriendGameReturnGaemInitiated()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitFriendGame();
            target.ConnectToFriendGame(gameInitInfo.GameId);

            var actual = target.CheckInitGameStatus(gameInitInfo);

            Assert.AreEqual(CheckGameInitStatus.GameInitiated, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void CheckInitGameStatusOnePlayerInitiatedFriendGameButWronPlayerIdReturnWrongPLayerId()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var gameInitInfo = target.InitFriendGame();
            gameInitInfo.PlayerId = Guid.NewGuid();
            var actual = target.CheckInitGameStatus(gameInitInfo);

            Assert.AreEqual(CheckGameInitStatus.WrongPlayerId, actual.Status);
        }
    }
}
