﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

using TicTacToe.Model.Model;
using TicTacToe.Model.Repositories;
using TicTacToe.Service.DataContracts;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Service.UnitTest.TicTacToeServiceTests
{
    [TestClass]
    public class GetLastStepTest
    {
        [TestMethod]
        public void GetLastStepNoGamesTest()
        {
            var service = TestTargetInitializer.BuildTicTacToeService();

            var step = new GameAndPlayerInfo { PlayerId = Guid.NewGuid(), GameId = Guid.NewGuid() };

            var actual = service.GetLastStep(step);

            Assert.IsNull(actual);
        }

        [TestMethod]
        public void GetLastStepNoPlayerTest()
        {
            var gameId = Guid.NewGuid();

            var games = new InMemoryGamesRepository();
            games.AddGame(TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), gameId));

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var game = new GameAndPlayerInfo { PlayerId = Guid.NewGuid(), GameId = gameId };

            var actual = service.GetLastStep(game);

            Assert.IsNull(actual);
        }

        [TestMethod]
        public void GetLastStepNoStepsTest()
        {
            var games = new InMemoryGamesRepository();
            var game = GetNewGame();
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var step = new GameAndPlayerInfo { PlayerId = game.Player1Id, GameId = game.GameId };

            var actual = service.GetLastStep(step);

            Assert.IsNull(actual);
        }

        [TestMethod]
        public void GetLastStepSuccessTest()
        {
            var games = new InMemoryGamesRepository();
            var game = GetNewGame();
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var p1 = new GameAndPlayerInfo { PlayerId = game.Player1Id, GameId = game.GameId };
            var p2 = new GameAndPlayerInfo { PlayerId = game.Player2Id, GameId = game.GameId };

            service.MakeStep(p1, 1, 1);

            var actual = service.GetLastStep(p2);

            Assert.AreEqual(game.Player1Id, actual.PlayerId);
            Assert.AreEqual(1, actual.OuterField);
            Assert.AreEqual(1, actual.InnerField);
        }

        private Game GetNewGame()
        {
            return TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());
        }
    }
}
