﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;

using System;
using TicTacToe.Model.Interfaces;
using TicTacToe.Model.Model;
using TicTacToe.Service.DataContracts;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Service.UnitTest.TicTacToeServiceTests
{
    [TestClass]
    public class InitFriendGameTest
    {
        [TestMethod, TestCategory("Service")]
        public void InitFriendGameFirstPlayerConnectedReturnWaitForOpponentStatus()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var actual = target.InitFriendGame();

            Assert.AreEqual(InitGameStatus.WaitForOpponent, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameFirstPlayerConnectedGameIdSet()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var actual = target.InitFriendGame();

            Assert.AreNotEqual(Guid.Empty, actual.GameId);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameFirstPlayerConnectedPlayerIdSet()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var actual = target.InitFriendGame();

            Assert.AreNotEqual(Guid.Empty, actual.PlayerId);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameFirstPlayerConnectedIsYourStepFirstTrue()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var actual = target.InitFriendGame();

            Assert.IsTrue(actual.IsYourStepFirst);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameSecondPlayerConnectedReturnWaitForOpponentStatus()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            target.InitFriendGame();
            var actualSecondPlayer = target.InitFriendGame();

            Assert.AreEqual(InitGameStatus.WaitForOpponent, actualSecondPlayer.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameThirdPlayerConnectedReturnWaitForOpponentStatus()
        {
            var target = TestTargetInitializer.BuildTicTacToeService();

            var initInfo = target.InitFriendGame();
            target.ConnectToFriendGame(initInfo.GameId);
            var actualThirdPlayer = target.InitFriendGame();

            Assert.AreEqual(InitGameStatus.WaitForOpponent, actualThirdPlayer.Status);
        }
        
        [TestMethod, TestCategory("Service")]
        public void InitFriendGameSessionIdIsNotSpecifiedNewSessionCreated()
        {
            var repository = new Mock<ISessionRepository>();
            var target = TestTargetInitializer.BuildTicTacToeService(sessionRepo: repository);

            target.InitFriendGame();
            
            repository.Verify(x => x.CreateTempSession(It.IsAny<Guid>()));
        }

        [TestMethod, TestCategory("Service")]
        public void InitGameSessionIdIsNotSpecifiedNewSessionIdReturned()
        {
            var repository = new Mock<ISessionRepository>();
            var target = TestTargetInitializer.BuildTicTacToeService(sessionRepo: repository);

            var actual = target.InitFriendGame();

            Assert.AreNotEqual(Guid.Empty, actual.SessionId);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameSessionIdIsSpecifiedButNotValidSessionNotValidStatusReturned()
        {
            var repository = new Mock<ISessionRepository>();
            repository.Setup(x => x.IsSessionValid(It.IsAny<Guid>())).Returns(false);

            var target = TestTargetInitializer.BuildTicTacToeService(sessionRepo: repository);

            var actual = target.InitFriendGame(Guid.NewGuid());

            Assert.AreEqual(InitGameStatus.SessionNotValid, actual.Status);
        }

        [TestMethod, TestCategory("Service")]
        public void InitFriendGameSessionIdIsSpecifiedExistingUserUsed()
        {
            var sessionId = Guid.NewGuid();
            var userId = Guid.NewGuid();
            var repository = new Mock<ISessionRepository>();
            repository.Setup(x => x.IsSessionValid(It.IsAny<Guid>())).Returns(true);
            repository.Setup(x => x.GetSessionById(It.Is<Guid>(y => y == sessionId)))
                .Returns(new Session { Id = sessionId, UserId = userId });

            var target = TestTargetInitializer.BuildTicTacToeService(sessionRepo: repository);

            var actual = target.InitFriendGame(sessionId);

            Assert.AreEqual(userId, actual.PlayerId);
            Assert.AreEqual(sessionId, actual.SessionId);
            repository.Verify(x => x.CreateTempSession(It.IsAny<Guid>()), Times.Never());
        }
    }
}
