﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

using TicTacToe.Model.Model;
using TicTacToe.Model.Repositories;
using TicTacToe.Service.DataContracts;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Service.UnitTest.TicTacToeServiceTests
{
    [TestClass]
    public class IsMyStepTest
    {
        [TestMethod]
        public void IsMyStepNoGamesTest()
        {
            var games = new InMemoryGamesRepository();
            
            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var step = new GameAndPlayerInfo() { PlayerId = Guid.NewGuid(), GameId = Guid.NewGuid() };

            var actual = service.IsMyStep(step);

            Assert.AreEqual(StepStatus.WaitForNextPlayerStep, actual);
        }

        [TestMethod]
        public void IsMyStepNoPlayerTest()
        {
            var gameId = Guid.NewGuid();

            var games = new InMemoryGamesRepository();
            games.AddGame(TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), gameId));

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var game = new GameAndPlayerInfo() { PlayerId = Guid.NewGuid(), GameId = gameId };

            var actual = service.IsMyStep(game);

            Assert.AreEqual(StepStatus.WaitForNextPlayerStep, actual);
        }

        [TestMethod]
        public void IsMyStepTrueTest()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var step = new GameAndPlayerInfo() { PlayerId = game.Player1Id, GameId = game.GameId };

            var actual = service.IsMyStep(step);

            Assert.AreEqual(StepStatus.YourTurn, actual);
        }

        [TestMethod]
        public void IsMyStepTrueSecondTest()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var p1 = new GameAndPlayerInfo() { PlayerId = game.Player1Id, GameId = game.GameId };
            var p2 = new GameAndPlayerInfo() { PlayerId = game.Player2Id, GameId = game.GameId };

            var actual = service.IsMyStep(p1);

            service.MakeStep(p1, 1, 1);

            actual = service.IsMyStep(p2);

            Assert.AreEqual(StepStatus.YourTurn, actual);
        }

        [TestMethod]
        public void IsMyStepFalseTest()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var step = new GameAndPlayerInfo() { PlayerId = game.Player2Id, GameId = game.GameId };

            var actual = service.IsMyStep(step);

            Assert.AreEqual(StepStatus.WaitForNextPlayerStep, actual);
        }

        [TestMethod]
        public void IsMyStepFalseSecondTest()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());
            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var p1 = new GameAndPlayerInfo() { PlayerId = game.Player1Id, GameId = game.GameId };
            var p2 = new GameAndPlayerInfo() { PlayerId = game.Player2Id, GameId = game.GameId };

            var actual = service.IsMyStep(p2);

            service.MakeStep(p1, 0, 0);

            actual = service.IsMyStep(p1);

            Assert.AreEqual(StepStatus.WaitForNextPlayerStep, actual);
        }

        [TestMethod]
        public void IsMyStepReturnTrueIfXWin()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());

            game.Field.Fill(@"x--
                              ---
                              x--");

            game.Field[3].Fill(@"x--
                                 ---
                                 x--");

            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var p1 = new GameAndPlayerInfo() { PlayerId = game.Player1Id, GameId = game.GameId };
            var p2 = new GameAndPlayerInfo() { PlayerId = game.Player2Id, GameId = game.GameId };

            service.MakeStep(p1, 3, 3);

            var actual = service.IsMyStep(p2);

            Assert.AreEqual(StepStatus.YourTurn, actual);
        }

        [TestMethod]
        public void IsMyStepReturnTrueIfOWin()
        {
            var games = new InMemoryGamesRepository();
            var game = TestTargetInitializer.BuildGame(Guid.NewGuid(), Guid.NewGuid(), Guid.NewGuid());

            game.Field.Fill(@"o--
                              ---
                              o--");

            game.Field[3].Fill(@"o--
                                 ---
                                 o--");

            games.AddGame(game);

            var service = TestTargetInitializer.BuildTicTacToeService(games);

            var p1 = new GameAndPlayerInfo() { PlayerId = game.Player1Id, GameId = game.GameId };
            var p2 = new GameAndPlayerInfo() { PlayerId = game.Player2Id, GameId = game.GameId };

            service.MakeStep(p1, 0, 3);
            service.MakeStep(p2, 3, 3);

            var actual = service.IsMyStep(p1);

            Assert.AreEqual(StepStatus.YourTurn, actual);
        }
    }
}
