﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;

using TicTacToe.Common.Security;
using TicTacToe.Model.Interfaces;
using TicTacToe.Model.Model;
using TicTacToe.Service.DataContracts;
using TicTacToe.UnitTest.Helpers;

namespace TicTacToe.Service.UnitTest.TicTacToeServiceTests
{
    [TestClass]
    public class RegisterUserTest
    {
        [TestMethod]
        public void RegisterUserAllValidAndUserNotExistsReturnTrue()
        {
            string login = "test";
            string password = "testp";

            var stubUserRepo = new Mock<IUserRepository>();
            stubUserRepo.Setup(item => item.CreateUser(It.IsAny<User>()));

            var target = TestTargetInitializer.BuildTicTacToeService(userRepo: stubUserRepo);

            var actual = target.RegisterUser(new UserRegistrationInfo { Email = login, Password = password });

            Assert.IsTrue(actual.IsSuccess);
        }

        [TestMethod]
        public void RegisterUserCreateUserCalled()
        {
            string login = "test";
            string password = "testp";

            var mockUserRepo = new Mock<IUserRepository>();

            var target = TestTargetInitializer.BuildTicTacToeService(userRepo: mockUserRepo);

            target.RegisterUser(new UserRegistrationInfo { Email = login, Password = password });

            mockUserRepo.Verify(item => 
                item.CreateUser(It.Is<User>(
                    user => user.Email == login && 
                            user.Login == login && 
                            new BCryptHashGenerator().IsValid(password, user.Password))));
        }

        [TestMethod]
        public void RegisterUserCreateUserThrowExceotionReturnFalse()
        {
            string login = "test";
            string password = "testp";

            var stubUserRepo = new Mock<IUserRepository>();
            stubUserRepo.Setup(item => item.CreateUser(It.IsAny<User>())).Throws<Exception>();

            var target = TestTargetInitializer.BuildTicTacToeService(userRepo: stubUserRepo);

            var actual = target.RegisterUser(new UserRegistrationInfo { Email = login, Password = password });

            Assert.IsFalse(actual.IsSuccess);
        }
    }
}
