﻿using System;
using System.ServiceModel;
using System.ServiceModel.Web;
using TicTacToe.Service.DataContracts;

namespace TicTacToe.Service.Interfaces
{
    [ServiceContract]
    public interface ITicTacToeService
    {
        [OperationContract]
        [WebInvoke(UriTemplate = "InitGame",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        GameInitInfo InitGame(Guid? sessionId);

        [OperationContract]
        [WebInvoke(UriTemplate = "InitFriendGame",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        GameInitInfo InitFriendGame(Guid? sessionId);

        [OperationContract]
        [WebInvoke(UriTemplate = "ConnectToFriendGame",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        GameInitInfo ConnectToFriendGame(Guid gameId, Guid? sessionId);

        [OperationContract]
        [WebInvoke(UriTemplate = "CheckInitGameStatus",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        CheckGameInitStatusResult CheckInitGameStatus(GameAndPlayerInfo gameAndPlayerInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "MakeStep",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        StepInfo MakeStep(GameAndPlayerInfo gameAndPlayerInfo, int outerField, int innerField);

        [OperationContract]
        [WebInvoke(UriTemplate = "IsMyStep",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json, 
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        StepStatus IsMyStep(GameAndPlayerInfo gameAndPlayerInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "GetLastStep",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json, 
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        StepInfo GetLastStep(GameAndPlayerInfo gameAndPlayerInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "IAmLive",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        void IAmLive(GameAndPlayerInfo gameAndPlayerInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "RegisterUser",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        RegistrationResult RegisterUser(UserRegistrationInfo registrationInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "Login",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        LoginResult Login(UserLoginInfo loginInfo);

        [OperationContract]
        [WebInvoke(UriTemplate = "Logout",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        void Logout(Guid sessionId);

        [OperationContract]
        [WebInvoke(UriTemplate = "GetUserInfo",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json,
            Method = "POST",
            BodyStyle = WebMessageBodyStyle.Wrapped)]
        UserInfoResult GetUserInfo(Guid? userId, Guid sessionId);
    }
}
