﻿using System;

using Moq;

using TicTacToe.Common.Log;
using TicTacToe.Model.Interfaces;
using TicTacToe.Model.Model;
using TicTacToe.Model.Repositories;
using TicTacToe.Service;
using TicTacToe.Service.Interfaces;

namespace TicTacToe.UnitTest.Helpers
{
    public static class TestTargetInitializer
    {
        public static TicTacToeService BuildTicTacToeService(
            IGamesRepository gameRepo = null,
            Mock<ISessionRepository> sessionRepo = null,
            Mock<IUserRepository> userRepo = null,
            Mock<IStatisitcsRepository> statRepo = null,
            int? playerDisconnectionTimeout = null)
        {
            var userStub = userRepo ?? new Mock<IUserRepository>();

            var statsStub = statRepo ?? new Mock<IStatisitcsRepository>();
            
            var sessionStub = sessionRepo ?? new Mock<ISessionRepository>();
            sessionStub
                .Setup(x => 
                    x.CreateTempSession(It.IsAny<Guid>()))
                    .Returns(new Session { Id = Guid.NewGuid() });

            return new TicTacToeService(
                gameRepo ?? new InMemoryGamesRepository(),
                new Mock<ILoggerFacade>().Object,
                new Mock<IConnectionPool>().Object,
                userStub.Object,
                statsStub.Object,
                sessionStub.Object,
                playerDisconnectionTimeout);
        }

        public static Game BuildGame(Guid? p1 = null, Guid? p2 = null, Guid? gameId = null, Mock<IStatisitcsRepository> statRepo = null)
        {
            var statsRepoStub = statRepo ?? new Mock<IStatisitcsRepository>();

            return new Game(p1 ?? Guid.NewGuid(), p2 ?? Guid.NewGuid(), gameId ?? Guid.NewGuid(), statsRepoStub.Object);
        }
    }
}