﻿using System.Web.Mvc;
using TicTacToe.Service.DataContracts;
using TicTacToe.WebServiceHost.Models;
using TicTacToe.WebServiceHost.TicTacToeService;

namespace TicTacToe.WebServiceHost.Controllers
{
    public class HomeController : BaseController
    {
        ////
        //// GET: /Home/Register

        public ActionResult Register()
        {
            return this.View();
        }

        ////
        //// POST: /Home/Register

        [HttpPost]
        public ActionResult Register(Registration user)
        {
            if (ModelState.IsValid)
            {
                using (var client = new TicTacToeServiceClient())
                {
                    var result =
                        client.RegisterUser(
                            new UserRegistrationInfo
                                {
                                    Email = user.Email,
                                    Password = user.Password,
                                    SessionId = this.CurrentSessionId
                                });

                    if (result.IsSuccess)
                    {
                        var userAuth = this.Auth.Login(user.Email, user.Password);
                        if (userAuth != null)
                        {
                            return this.RedirectToAction("Index", "Game");
                        }
                        
                        return this.RedirectToAction("SignIn", "Home");
                    }
                }

                ModelState["Email"].Errors.Add("User already exist");
            }

            return this.View();
        }

        public PartialViewResult UserLogin()
        {
            return this.PartialView(this.CurrentUser);
        }

        public ActionResult Logout()
        {
            this.Auth.Logout();
            return this.RedirectToAction("Index", "Game");
        }

        ////
        //// GET: /Home/SignIn
        public ActionResult SignIn()
        {
            return this.View();
        }

        ////
        //// POST: /Home/SignIn
        [HttpPost]
        public ActionResult SignIn(Registration user)
        {
            if (ModelState.IsValid)
            {
                var userSession = this.Auth.Login(user.Email, user.Password);
                if (userSession != null)
                {
                    return this.RedirectToAction("Index", "Game");
                }

                ModelState["Password"].Errors.Add("Wrong password");
            }

            return this.View();
        }
    }
}
