﻿var client = {
    sessionId: "",
    gameId: "",
    playerId: "",
    opponentId: "",

    log: function(str) {
        //$('#log').html($('#log').html() + "<br />" + str);
    },

    getGameAndPlayerInfo: function() {
        return {
            gameAndPlayerInfo: {
                GameId: client.gameId,
                PlayerId: client.playerId,
                SessionId: client.sessionId
            }
        };
    },

    iAmLiveInterval: null,

    initGame: function() {
        client.log("Initiating game...");
        client.isXStep = true;
        interaction.setStatus("Connecting...");
        var data = null;
        if (client.sessionId != "") {
            data = JSON.stringify({ sessionId: client.sessionId });
        }

        $.ajax({
            url: "TicTacToe.svc/rest/InitGame",
            type: "POST",
            dataType: "json",
            contentType: "application/json",
            data: data,
            success: function(result) {
                client.gameId = result.InitGameResult.GameId;
                chat.connect(client.gameId);
                client.playerId = result.InitGameResult.PlayerId;
                client.sessionId = result.InitGameResult.SessionId;
                if (result.InitGameResult.Status == 0) {
                    // wait for opponent.
                    interaction.setSessionId(client.sessionId);
                    client.waitForOpponent();
                } else if (result.InitGameResult.Status == 1) {
                    // game created.

                    client.opponentId = result.InitGameResult.OpponentId;
                    interaction.opponentConnected();
                    client.isXStep = result.InitGameResult.IsYourStepFirst;
                    var x = interaction.myStep;
                    var o = interaction.opponentStep;
                    interaction.myStep = client.isXStep ? x : o;
                    interaction.opponentStep = client.isXStep ? o : x;
                    if (result.InitGameResult.IsYourStepFirst) {
                        client.enableFields(-1);
                    } else {
                        client.waitForOpponentStep();
                    }
                    interaction.setSessionId(client.sessionId);
                    client.iAmLiveInterval = setInterval(client.iAmLive, 1000);
                } else if (result.InitGameResult.Status == 2) {
                    interaction.removeSessionId();
                } else {
                    client.showError();
                }
            },
            error: function() {
                client.showError();
            }
        });
    },

    initFriendGame: function() {
        client.log("Initiating game...");
        client.isXStep = true;
        interaction.setStatus("Connecting...");

        var data = null;
        if (client.sessionId != "") {
            data = JSON.stringify({ sessionId: client.sessionId });
        }

        $.ajax({
            url: "TicTacToe.svc/rest/InitFriendGame",
            type: "POST",
            dataType: "json",
            contentType: "application/json",
            data: data,
            success: function(result) {
                client.gameId = result.InitFriendGameResult.GameId;
                chat.connect(client.gameId);
                client.playerId = result.InitFriendGameResult.PlayerId;
                client.sessionId = result.InitFriendGameResult.SessionId;
                if (result.InitFriendGameResult.Status == 0) {
                    // wait for opponent.
                    interaction.showFriendLink(client.gameId);
                    client.waitForOpponent();
                    interaction.setSessionId(client.sessionId);
                    client.iAmLiveInterval = setInterval(client.iAmLive, 1000);
                } else if (result.InitFriendGameResult.Status == 2) {
                    interaction.removeSessionId();
                } else {
                    client.showError();
                }
            },
            error: function() {
                client.showError();
            }
        });
    },

    connectToFriendGame: function(gameId) {
        client.log("Connection to friend game...");
        client.isXStep = true;
        interaction.setStatus("Connecting...");
        var data = { gameId: gameId };
        if (client.sessionId != "") {
            data.sessionId = client.sessionId;
        }

        data = JSON.stringify(data);
        $.ajax({
            url: "TicTacToe.svc/rest/ConnectToFriendGame",
            type: "POST",
            dataType: "json",
            contentType: "application/json",
            data: data,
            success: function(result) {
                client.gameId = result.ConnectToFriendGameResult.GameId;
                chat.connect(client.gameId);
                client.playerId = result.ConnectToFriendGameResult.PlayerId;
                client.sessionId = result.ConnectToFriendGameResult.SessionId;
                if (result.ConnectToFriendGameResult.Status == 1) {
                    // game created.
                    client.opponentId = result.ConnectToFriendGameResult.OpponentId;
                    interaction.opponentConnected();
                    client.isXStep = result.ConnectToFriendGameResult.IsYourStepFirst;
                    var x = interaction.myStep;
                    var o = interaction.opponentStep;
                    interaction.myStep = client.isXStep ? x : o;
                    interaction.opponentStep = client.isXStep ? o : x;
                    if (result.ConnectToFriendGameResult.IsYourStepFirst) {
                        client.enableFields(-1);
                    } else {
                        client.waitForOpponentStep();
                    }
                    interaction.setSessionId(client.sessionId);
                    client.iAmLiveInterval = setInterval(client.iAmLive, 1000);
                } else if (result.ConnectToFriendGameResult.Status == 2) {
                    interaction.removeSessionId();
                } else if (result.ConnectToFriendGameResult.Status == 3) {
                    // game not exists
                    client.friendGameIsNotExist();
                }
            },
            error: function() {
                client.showError();
            }
        });
    },

    isXStep: true,

    waitForOpponent: function() {
        client.log("Waiting for opponent...");
        interaction.setStatus("Waiting for opponent...");
        var data = JSON.stringify(client.getGameAndPlayerInfo());
        $.ajax({
            url: "TicTacToe.svc/rest/CheckInitGameStatus",
            type: "POST",
            dataType: "json",
            data: data,
            contentType: "application/json",
            success: function(result) {
                if (result.CheckInitGameStatusResult.Status == 0) {
                    setTimeout(client.waitForOpponent, 1000);
                } else if (result.CheckInitGameStatusResult.Status == 3) {
                    // game initiated
                    client.opponentId = result.CheckInitGameStatusResult.OpponentId;
                    interaction.opponentConnected();
                    if (client.isXStep) {
                        client.enableFields(-1);
                    } else {
                        client.waitForOpponentStep();
                    }

                    client.iAmLiveInterval = setInterval(client.iAmLive, 1000);
                } else {
                    // game or player id is wrong
                    client.showError();
                }
            },
            error: function() {
                client.showError();
            }
        });
    },

    waitForOpponentStep: function () {
        client.log("Waiting for opponent step...");
        interaction.setStatus("Waiting for opponent's step...");
        var data = JSON.stringify(client.getGameAndPlayerInfo());
        $.ajax({
            url: "TicTacToe.svc/rest/IsMyStep",
            type: "POST",
            dataType: "json",
            data: data,
            contentType: "application/json",
            success: function (result) {
                if (result.IsMyStepResult == 0) {
                    // keep waiting
                    setTimeout(client.waitForOpponentStep, 1000);
                } else if (result.IsMyStepResult == 10) {
                    // your turn
                    client.showOpponentStep();
                    interaction.deleteBlock();
                } else if (result.IsMyStepResult == 5) {
                    // you win.
                    client.winTheGame();
                } else if (result.IsMyStepResult == 8) {
                    // draw
                    client.drawTheGame();
                } else if (result.IsMyStepResult == 9) {
                    // opponent disconected
                    client.opponentDisconected();
                } else {
                    client.showError();
                }
            },
            error: function () {
                client.showError();
            }
        });
    },
    
    showOpponentStep: function () {
        client.log("Getting oppoent last step...");
        var data = JSON.stringify(client.getGameAndPlayerInfo());
        $.ajax({
            url: "TicTacToe.svc/rest/GetLastStep",
            type: "POST",
            dataType: "json",
            data: data,
            contentType: "application/json",
            success: function (result) {
                // outer fiels is not empty and correct outer field.
                if (result.GetLastStepResult.OuterFieldStatus != 0) {
                    interaction.drawOuterFieldState(result.GetLastStepResult.OuterFieldStatus, result.GetLastStepResult.OuterField);
                }
                
                client.drawOpponentStep(result.GetLastStepResult.OuterField, result.GetLastStepResult.InnerField);
                client.enableFields(result.GetLastStepResult.InnerField);
                
                if (result.GetLastStepResult.GameStatus == 5) {
                    // draw 
                    client.drawTheGame();
                } else if (result.GetLastStepResult.GameStatus == (client.isXStep ? 3 : 2)) {
                    // opponent win
                    client.lostTheGame();
                } else if (result.GetLastStepResult.GameStatus == 4) {
                    // opponet disconected
                    client.opponentDisconected();
                } 
            },
            error: function () {
                client.showError();
            }
        });
    },
    
    iAmLive: function () {
        var data = JSON.stringify(client.getGameAndPlayerInfo());
        $.ajax({
            url: "TicTacToe.svc/rest/IAmLive",
            type: "POST",
            dataType: "json",
            data: data,
            contentType: "application/json"
        });
    },
    
    makeStep: function (outerIndex, innerIndex) {
        client.log("Make step in to " + outerIndex + " " + innerIndex);
        var data = client.getGameAndPlayerInfo();
        data.outerField = outerIndex;
        data.innerField = innerIndex;
        data = JSON.stringify(data);
        
        $.ajax({
            url: "TicTacToe.svc/rest/MakeStep",
            type: "POST",
            dataType: "json",
            data: data,
            contentType: "application/json",
            success: function (result) {
                // outer fiels is not empty and correct outer field.
                if (result.MakeStepResult.OuterFieldStatus != 0
                    && result.MakeStepResult.OuterField == outerIndex) {
                    interaction.drawOuterFieldState(result.MakeStepResult.OuterFieldStatus, outerIndex);
                }
                if (result.MakeStepResult.StepStatus == 0) {
                    interaction.block(outerIndex);
                    client.waitForOpponentStep();
                } else if (result.MakeStepResult.StepStatus == 5) {
                    client.winTheGame();
                } else if (result.MakeStepResult.StepStatus == 8) {
                    client.drawTheGame();
                } else if (result.MakeStepResult.StepStatus == 9) {
                    client.opponentDisconected();
                } else {
                    client.showError();
                }
            },
        });
    },
    
    enableFields: function (outerIndex) {
        client.log("Waiting for your step...");
        interaction.setStatus("Make your step");
        interaction.deleteDisable();
        interaction.notifyFavicon(client.isXStep ? 'X' : 'O');
        if (outerIndex != -1) {
            interaction.disable(outerIndex);
        }
    },
    
    winTheGame: function () {
        // show win status
        client.log("You win...");
        interaction.winGame();
        client.endTheGame();
    },
    
    lostTheGame: function() {
        client.log("You lost...");
        interaction.lostGame();
        client.endTheGame();
    },

    drawTheGame: function () {
        // show draw status
        client.log("Draw...");
        interaction.drawGame();
        client.endTheGame();
    },
    
    friendGameIsNotExist: function () {
        client.gameId = "";
        client.playerId = "";
        client.sessionId = "";
        interaction.friendGameIsNotExist();
        client.endTheGame();
    },

    drawOpponentStep: function (outerIndex, innerIndex) {
        // draw opponent step
        interaction.deleteHighlightLastStep("opponent");
        interaction.showOpponentStep(outerIndex, innerIndex);
        interaction.highlightLastStep(outerIndex, innerIndex, "opponent");
        client.log("Opponent make step in to " + outerIndex + " " + innerIndex);
    },

    opponentDisconected: function() {
        client.log("Opponent disconected...");
        client.winTheGame();
    },
    
    endTheGame: function () {
        clearInterval(client.iAmLiveInterval);
        interaction.disable(-1);
        interaction.endTheGame();
    },

    showError: function () {
        noty({ text: 'Oops, something goes wrong. Try again later...', type: 'alert' });
    },
    
    getUserInfo: function (userId, success, error, panel) {
        client.log("Getting user info ...");

        var data = JSON.stringify({ userId: userId == "" ? null : userId, sessionId: client.sessionId });
        
        $.ajax({
            url: "TicTacToe.svc/rest/GetUserInfo",
            type: "POST",
            dataType: "json",
            contentType: "application/json",
            data: data,
            success: function (result) {
                if (result.GetUserInfoResult.IsSuccess) {
                    success({
                        email: result.GetUserInfoResult.Email,
                        playedGameCount: result.GetUserInfoResult.PlayedGameCount,
                        avatarHash: result.GetUserInfoResult.AvatarHash,
                        wonGameCount: result.GetUserInfoResult.WonGameCount,
                        lostGameCount: result.GetUserInfoResult.LostGameCount,
                        drawGameCount: result.GetUserInfoResult.DrawGameCount,
                        leaveGameCount: result.GetUserInfoResult.LeaveGameCount
                    }, panel);
                } else {
                    error();
                }
            },
            error: function () {
                error();
            }
        });
    }
}