﻿/*  
    Module for interface interaction.
*/

var interaction = {
    startButtonId: "#start",
    startWithFriendButtonId: "#startWithFriendButton",
    statusId: "#status",
    friendLinkId: "#friendLink",
    friendLinkContainer: "#friendLinkContainer",
    sessionId: "#sessionId",
    userId: "#userId",
    sendMessageButtonId: "#sendMessage",
    chatMessageId: "#chatMessage",

    init: function() {
        interaction.disable(-1);

        var user = $(userId).val();
        var session = $(sessionId).val();
        if (user != "") {
            client.playerId = user;
        }
        if (session != "") {
            client.sessionId = session;
            interaction.showCurrentUserInfoBlock();
        }

        $(interaction.startButtonId).click(function() {
            interaction.collapseButtons();
            interaction.expandStatusBar();
            interaction.clearFields();
            chat.leave();
            client.initGame();
        });

        $(interaction.sendMessageButtonId).click(function () {
            interaction.sendMessage();
        });

        $(interaction.chatMessageId).keypress(function (e) {
            if (e.which == 13) {
                interaction.sendMessage();
                e.preventDefault();
            }
        });

        $(interaction.startWithFriendButtonId).click(function() {
            interaction.collapseButtons();
            interaction.expandStatusBar();
            interaction.clearFields();
            chat.leave();
            client.initFriendGame();
        });

        $('.innerField').click(function() {
            if (!$(this).hasClass('disabled') && !$(this).hasClass('used') && !$(this).hasClass('block')) {
                var outer = $(this).data("outerindex");
                var inner = $(this).data("innerindex");
                interaction.deleteHighlightLastStep("player");
                client.makeStep(outer, inner);
                interaction.highlightLastStep(outer, inner, "player");
                $(this).addClass(interaction.myStep);
                $(this).addClass('used');
                interaction.disable(-1); // disable all
                interaction.notifyFavicon();
            } else {
                client.log('You can not make this step');
            }
        });
    },

    showCurrentUserInfoBlock: function() {
        client.getUserInfo(client.playerId, interaction.drawUserInfoBlock, interaction.removeSessionId, ".left");
    },

    showOpponentUserInfoBlock: function() {
        $('.right').animate({ width: "200px" }, 300);
        client.getUserInfo(client.opponentId, interaction.drawUserInfoBlock, interaction.removeSessionId, ".right");
    },

    drawUserInfoBlock: function(userInfo, panel) {
        var src = $(panel + " .avatarImg").attr("src");
        src = src.replace("{hash}", userInfo.avatarHash);
        $(panel + " .avatarImg").attr("src", src);

        $(panel + " .email").text(userInfo.email);
        $(panel + " .played").text(userInfo.playedGameCount);
        $(panel + " .win").text(userInfo.wonGameCount);
        $(panel + " .lost").text(userInfo.lostGameCount);
        $(panel + " .draw").text(userInfo.drawGameCount);
        $(panel + " .leave").text(userInfo.leaveGameCount);
    },
    
    incrementStatInPanel: function(stat, panel) {
        var val = parseInt($(panel + " " + stat).text());
        $(panel + " " + stat).text(val + 1);
    },

    setSessionId: function(sessionId) {
        interaction.setCookie("sessionId", sessionId, 1);
    },

    removeSessionId: function() {
        client.sessionId = "";
        interaction.setCookie("sessionId", "", 1);
    },

    setCookie: function setCookie(name, value, exdays) {
        var exdate = new Date();
        exdate.setDate(exdate.getDate() + exdays);
        var cvalue = escape(value) + ((exdays == null) ? "" : "; expires=" + exdate.toUTCString());
        document.cookie = name + "=" + cvalue;
    },

    statusBarSavedProps: null,

    expandStatusBar: function() {
        interaction.statusBarSavedProps = {
            "margin": $(interaction.statusId).css("margin"),
            "display": $(interaction.statusId).css("display"),
            "width": $(interaction.statusId).css("width")
        };

        $(interaction.statusId).css({ "margin": "0 auto" });
        $(interaction.statusId).css({ "display": "block" });
        $(interaction.statusId).animate({ width: "455px" }, 300);
    },

    collapseStatusBar: function(callback) {
        $(interaction.statusId).animate({ width: interaction.statusBarSavedProps.width }, 150, function() {
            $(interaction.statusId).css({ "display": interaction.statusBarSavedProps.display });
            $(interaction.statusId).css({ "margin": 150 });
            callback();
            $(interaction.statusId).css({ "margin": interaction.statusBarSavedProps.margin });

        });
    },

    collapseButtons: function() {
        $(interaction.startButtonId).hide();
        $(interaction.startWithFriendButtonId).hide();
    },

    expandButtons: function() {
        $(interaction.startButtonId).show();
        $(interaction.startWithFriendButtonId).show();
    },

    endTheGame: function() {
        $(interaction.startButtonId).removeAttr('disabled');
        $(interaction.startWithFriendButtonId).removeAttr('disabled');
    },
    
    // can make move only to sended field
    disable: function(outerIndex) {
        var allInnerFieldUser = true;

        var selector = '#innerTable' + outerIndex + ' > tbody > tr > td.innerField';
        $(selector).each(function() {
            if (!$(this).hasClass('used')) {
                allInnerFieldUser = false;
            }
        });

        if (allInnerFieldUser && outerIndex != -1) {
            interaction.deleteDisable();
        } else {
            $('.innerField').each(function() {
                if ($(this).data("outerindex") != outerIndex) {
                    $(this).addClass('disabled');
                }
            });
        }
    },

    deleteDisable: function() {
        $('.disabled').each(function() {
            $(this).removeClass('disabled');
        });
    },
    
    // highligh last step
    // userType -> player, opponent
    highlightLastStep: function (outerIndex, innerIndex, userType) {
        var field = $("#innerField" + outerIndex + "" + innerIndex);
        field.addClass(userType + 'sLastStep');
    },

    // delete last step
    // userType -> player, opponent
    deleteHighlightLastStep: function (userType) {
        $('.' + userType + 'sLastStep').each(function() {
            $(this).removeClass(userType + 'sLastStep');
        });
    },
    
    // user can not make step in fields, while opponent makes step
    block: function(outerIndex) {
        $('.innerField').each(function() {
            if ($(this).data("outerindex") == outerIndex) {
                $(this).addClass('block');
            }
        });
    },

    deleteBlock: function() {
        $('.block').each(function() {
            $(this).removeClass('block');
        });
    },

    winGame: function() {
        interaction.collapseStatusBar(function() {
            interaction.expandButtons();
        });

        interaction.incrementStatInPanel('.lost', '.right');
        interaction.incrementStatInPanel('.win', '.left');
        interaction.setStatus("You win :)");
    },

    setStatus: function(text) {
        $('#status').text(text);
    },

    lostGame: function() {
        interaction.collapseStatusBar(function() {
            interaction.expandButtons();
        });

        interaction.incrementStatInPanel('.lost', '.left');
        interaction.incrementStatInPanel('.win', '.right');
        interaction.setStatus("You lost :(");
    },

    drawGame: function() {
        interaction.collapseStatusBar(function() {
            interaction.expandButtons();
        });

        interaction.incrementStatInPanel('.draw', '.left');
        interaction.incrementStatInPanel('.draw', '.right');
        interaction.setStatus("Draw :|");
    },

    opponentConnected: function() {
        $(interaction.friendLinkContainer).animate({ height: "0" }, 200, function() {
            $(interaction.friendLinkContainer).css("display", "none");
        });
        
        interaction.incrementStatInPanel('.played', '.left');
        interaction.incrementStatInPanel('.played', '.right');
        interaction.showOpponentUserInfoBlock();
        $('.chat-container').fadeIn();
        $('.chat-content').html('');
    },

    drawOuterFieldState: function(status, index) {
        var id = '#innerTable' + index;
        switch (status) {
        case 2:
// o
            $(id).addClass("o-bg");
            break;
        case 1:
// x
            $(id).addClass("x-bg");
            break;
        case 3:
// draw
            $(id).addClass("draw-bg");
            break;
        }
    },

    showOpponentStep: function(outerIndex, innerIndex) {
        var field = $("#innerField" + outerIndex + "" + innerIndex);
        field.addClass(interaction.opponentStep);
        field.addClass('used');
    },

    clearFields: function() {
        $('.innerField').each(function() {
            $(this).removeClass('o-bg-white');
            $(this).removeClass('x-bg-white');
            $(this).removeClass('used');
        });

        $('.innerTable').each(function() {
            $(this).removeClass('o-bg');
            $(this).removeClass('x-bg');
            $(this).removeClass('draw-bg');
        });

        interaction.myStep = "x-bg-white";
        interaction.opponentStep = "o-bg-white";
    },

    friendGameIsNotExist: function() {
        interaction.collapseStatusBar(function() {
            interaction.expandButtons();
        });

        interaction.setStatus("Game not found");
    },

    showFriendLink: function(gameId) {
        $(interaction.friendLinkContainer).css("display", "block");
        $(interaction.friendLinkContainer).css("height", "0");
        $(interaction.friendLinkContainer).animate({ height: "90" }, 200);

        var full = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : '');
        $("#friendLink").attr('value', full + "/?gameId=" + gameId);
        $("#friendLink").focus();
        $("#friendLink").select();
    },

    changeFavicon: function(src) {
        var link = document.createElement('link'),
            oldLink = document.getElementById('dynamic-favicon');
        link.id = 'dynamic-favicon';
        link.rel = 'shortcut icon';
        link.href = src;
        if (oldLink) {
            document.head.removeChild(oldLink);
        }
        document.head.appendChild(link);
    },

    iconBlinkingInverval: null,
    
    startIconBlinking: function(letter) {
        window.clearInterval(this.iconBlinkingInverval);
        var invetredImage = true;
        this.iconBlinkingInverval = setInterval(function () {
            var image = letter + "_ico" + (invetredImage ? "_inv" : "") + ".ico";
            interaction.changeFavicon(window.location.href + "Content/favicon/" + image);
            invetredImage = !invetredImage;
        }, 500);
    },

    notifyFavicon: function(type) {
        switch (type) {
            case 'X':
                this.startIconBlinking("x");
                break;
            case 'O':
                this.startIconBlinking("o");
                break;
            default:
                window.clearInterval(this.iconBlinkingInverval);
                this.changeFavicon(window.location.href + "Content/favicon/favicon.ico");
                break;
        }
        
    },
    
    addYourMessageToChat: function(message) {
        interaction.addMessageToChat("your", message, interaction.myStep);
        $(interaction.chatMessageId).val('').focus();
    },
    
    addOpponentMessageToChat: function(message) {
        interaction.addMessageToChat("opponent", message, interaction.opponentStep);
    },
    
    addMessageToChat: function(player, message, step) {
        var newMessage = '<p class="' + player + ' message ' + step + ' hiddenMessage"></p>';
        $('.chat-content').html(newMessage + $('.chat-content').html());
        $('.hiddenMessage').text(message);
        $('.hiddenMessage').fadeIn();
        $('.hiddenMessage').removeClass('hiddenMessage');
    },
    
    sendMessage: function () {
        var message = $(interaction.chatMessageId).val();
        if (message != '') {
            chat.sendMessage(message);
            interaction.addYourMessageToChat(message);
        }
    },

    myStep: "x-bg-white",
    opponentStep: "o-bg-white"
};
